import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Lights;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;

/*
 * author: Ash
 * 
 * Created on 30.11.2003
 * 
 * This is the second part of the jPCT Tutorial. Its very similar to the first tutorial,
 * but here we will use the hardware renderer. So the main code remains the same, i will
 * add comments for new code.
 * We don't have to create a frame in this example (this handles lwjgl), but we have to
 * deactivate the software renderer. One problem is that lwjg has its own way to handle
 * window- and key-events, so in this tutorial it is not possible to close the window 
 * (we have to define a kind of mapping between our application and lwjgl; an example
 * for this will follow in a later tutorial)  
 * 
 * Hint: For LWJGL use -Djava.library.path=d:/jpctapi/lib/lwjgl-0.7/ as vm argument   
 */

public class JPCTTutorial2 {

	private Object3D cube = null;
	private World theWorld = null;
	private Camera camera = null;
	private FrameBuffer buffer = null;
		
	private int width = 640;
	private int height = 480;
	
	private Frame mainFrame = null;
	private Graphics frameGraphics = null;
	private int titleBarHeight = 0;
	private int leftBorderWidth = 0;
	
	private boolean exit = false;
	
	public JPCTTutorial2() {
		theWorld = new World();
		
		theWorld.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		theWorld.setAmbientLight(20, 20, 20);
		
		theWorld.addLight(new SimpleVector(10.0f, -10.0f, -30.0f), Color.white);
		
		cube = Primitives.getCube(10.0f);
		theWorld.addObject(cube);
		
		camera = theWorld.getCamera();
		camera.moveCamera(Camera.CAMERA_MOVEOUT, 60);
	
		theWorld.buildAllObjects();
		
		gameloop();
	}
	
	private void gameloop() {
		buffer = new FrameBuffer(width, height, FrameBuffer.SAMPLINGMODE_NORMAL);
		
		// Deactivate the software renderer
		buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
		
		// Activate our OpenGL renderer
		buffer.enableRenderer(IRenderer.RENDERER_OPENGL, IRenderer.MODE_OPENGL);
		buffer.optimizeBufferAccess();
		
		while (!exit) {
			buffer.clear();
			theWorld.renderScene(buffer);
			theWorld.draw(buffer);
			buffer.update();
			// Show buffer content to lwjgl-window.
			buffer.displayGLOnly();
		}
		System.exit(0);
	}
	
	public static void main(String[] args) {
		JPCTTutorial2 helloWorld = new JPCTTutorial2();
	}
}
