import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Lights;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;


public class KeyBoardTest {

	
	private Object3D cube = null;
	private World theWorld = null;
	private Camera camera = null;
	private FrameBuffer buffer = null;
		
	private static int width = 640;
	private static int height = 480;
	
	private static Frame mainFrame = null;
	private Graphics frameGraphics = null;
	private static int titleBarHeight = 0;
	private static int leftBorderWidth = 0;
	
	private boolean exit = false;
	private boolean softRenderer = true;
	private boolean switchRenderer = false;
		
	TestWindow test = null;
	static JPCTKeyboard testw = null;
	
	public KeyBoardTest() {
		theWorld = new World();
		JPCTKeyboard.create();
				
		theWorld.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		
		theWorld.setAmbientLight(20, 20, 20);
		
		theWorld.addLight(new SimpleVector(10.0f, -10.0f, -30.0f), Color.red);
		
		cube = Primitives.getCube(10.0f);
		theWorld.addObject(cube);
		
		camera = theWorld.getCamera();
		camera.moveCamera(Camera.CAMERA_MOVEOUT, 60);
		
		theWorld.buildAllObjects();
		
		initializeFrame();
		test = new TestWindow(this);
		mainFrame.requestFocus();
				
		gameloop();
	}
	
	private void initializeFrame() {
		mainFrame = new Frame();
		mainFrame.setTitle("jPCT Tutorial #1: Hello World");
		mainFrame.pack();
		
		Insets insets = mainFrame.getInsets();
		titleBarHeight=insets.top;
		leftBorderWidth=insets.left;
		mainFrame.setSize(width+leftBorderWidth+insets.right, height+titleBarHeight+insets.bottom);
		mainFrame.addKeyListener(JPCTKeyboard.getInstance());
		
		mainFrame.setResizable(false);
		
		mainFrame.show();
		
		frameGraphics = mainFrame.getGraphics();
	}			
	
	private void gameloop() {
		
		buffer = new FrameBuffer(width, height, FrameBuffer.SAMPLINGMODE_NORMAL);
		buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);		
		
		buffer.optimizeBufferAccess();
		
		while (!exit) {
			JPCTKeyboard.keybPoll();
			if (switchRenderer)
				changeRenderer();
			if (JPCTKeyboard.isKeyPressed(JPCTKeyboard.JPCTKEY_UP))
				camera.moveCamera(Camera.CAMERA_MOVEIN, 0.1f);
			if (JPCTKeyboard.isKeyPressed(JPCTKeyboard.JPCTKEY_DOWN))
				camera.moveCamera(Camera.CAMERA_MOVEOUT, 0.1f);	
			if (JPCTKeyboard.isKeyPressed(JPCTKeyboard.JPCTKEY_LEFT))
				camera.rotateCameraY(-0.01f);	
			if (JPCTKeyboard.isKeyPressed(JPCTKeyboard.JPCTKEY_RIGHT))
				camera.rotateCameraY(0.01f);										
			buffer.clear();
			theWorld.renderScene(buffer);
			theWorld.draw(buffer);
			buffer.update();
			if (buffer.usesRenderer(IRenderer.RENDERER_OPENGL))
				buffer.displayGLOnly();
			else
				buffer.display(frameGraphics, leftBorderWidth, titleBarHeight);
		}
		System.exit(0);
	}
	
	public static Point getPosition() {
		return mainFrame.getLocation();
	}
	
	public static Dimension getSize() {
		return mainFrame.getSize();
	}
	
	public void changeRenderer() {
		if (!softRenderer) {
			mainFrame.hide();
			buffer.enableRenderer(IRenderer.RENDERER_OPENGL, IRenderer.MODE_OPENGL);
			buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
			JPCTKeyboard.setSoftRenderer(false);
			JPCTKeyboard.createLWJGLKeyboard();
			softRenderer = false;
			switchRenderer = false;
		}
		else {
			buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
			buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);
			JPCTKeyboard.setSoftRenderer(true);
			JPCTKeyboard.destroyLWJGLKeyboard();
			mainFrame.hide();
			mainFrame.dispose();
			softRenderer = true;	
			initializeFrame();
			switchRenderer = false;
		}	
	}
	
	public void setSoftRenderer(boolean softRenderer) {
		this.softRenderer = softRenderer;
		switchRenderer = true;
	}
	
	public void setExit() {
		this.exit = true;
	}
	
	public static void main(String[] args) {
		KeyBoardTest helloWorld = new KeyBoardTest();
	}
}
